@@ -27,10 +27,10 @@ import scala.reflect._
 
 import com.google.common.io.Files
 import org.apache.commons.io.IOUtils
-import org.apache.log4j.{Appender, Level, Logger}
-import org.apache.log4j.spi.LoggingEvent
+import org.apache.logging.log4j._
+import org.apache.logging.log4j.core.{Appender, LogEvent, Logger}
 import org.mockito.ArgumentCaptor
-import org.mockito.Mockito.{atLeast, mock, verify}
+import org.mockito.Mockito.{atLeast, mock, verify, when}
 import org.scalatest.BeforeAndAfter
 
 import org.apache.spark.{SparkConf, SparkFunSuite}
@@ -222,14 +222,15 @@ class FileAppenderSuite extends SparkFunSuite with BeforeAndAfter with Logging {
       // assert(appender.getClass === classTag[ExpectedAppender].getClass)
       assert(appender.getClass.getSimpleName ===
         classTag[ExpectedAppender].runtimeClass.getSimpleName)
-      if (appender.isInstanceOf[RollingFileAppender]) {
-        val rollingPolicy = appender.asInstanceOf[RollingFileAppender].rollingPolicy
-        val policyParam = if (rollingPolicy.isInstanceOf[TimeBasedRollingPolicy]) {
-          rollingPolicy.asInstanceOf[TimeBasedRollingPolicy].rolloverIntervalMillis
-        } else {
-          rollingPolicy.asInstanceOf[SizeBasedRollingPolicy].rolloverSizeBytes
-        }
-        assert(policyParam === expectedRollingPolicyParam)
+      appender match {
+        case rfa: RollingFileAppender =>
+          val rollingPolicy = rfa.rollingPolicy
+          val policyParam = rollingPolicy match {
+            case timeBased: TimeBasedRollingPolicy => timeBased.rolloverIntervalMillis
+            case sizeBased: SizeBasedRollingPolicy => sizeBased.rolloverSizeBytes
+          }
+          assert(policyParam === expectedRollingPolicyParam)
+        case _ => // do nothing
       }
       testOutputStream.close()
       appender.awaitTermination()
@@ -274,10 +275,13 @@ class FileAppenderSuite extends SparkFunSuite with BeforeAndAfter with Logging {
   test("file appender async close stream abruptly") {
     // Test FileAppender reaction to closing InputStream using a mock logging appender
     val mockAppender = mock(classOf[Appender])
-    val loggingEventCaptor = ArgumentCaptor.forClass(classOf[LoggingEvent])
+    when(mockAppender.getName).thenReturn("appender")
+    when(mockAppender.isStarted).thenReturn(true)
+
+    val loggingEventCaptor = ArgumentCaptor.forClass(classOf[LogEvent])
 
     // Make sure only logging errors
-    val logger = Logger.getRootLogger
+    val logger = LogManager.getRootLogger().asInstanceOf[Logger]
     val oldLogLevel = logger.getLevel
     logger.setLevel(Level.ERROR)
     try {
@@ -294,22 +298,26 @@ class FileAppenderSuite extends SparkFunSuite with BeforeAndAfter with Logging {
       appender.awaitTermination()
 
       // If InputStream was closed without first stopping the appender, an exception will be logged
-      verify(mockAppender, atLeast(1)).doAppend(loggingEventCaptor.capture)
+      verify(mockAppender, atLeast(1)).append(loggingEventCaptor.capture)
       val loggingEvent = loggingEventCaptor.getValue
-      assert(loggingEvent.getThrowableInformation !== null)
-      assert(loggingEvent.getThrowableInformation.getThrowable.isInstanceOf[IOException])
+      assert(loggingEvent.getThrown !== null)
+      assert(loggingEvent.getThrown.isInstanceOf[IOException])
     } finally {
       logger.setLevel(oldLogLevel)
+      logger.removeAppender(mockAppender)
     }
   }
 
   test("file appender async close stream gracefully") {
     // Test FileAppender reaction to closing InputStream using a mock logging appender
     val mockAppender = mock(classOf[Appender])
-    val loggingEventCaptor = ArgumentCaptor.forClass(classOf[LoggingEvent])
+    when(mockAppender.getName).thenReturn("appender")
+    when(mockAppender.isStarted).thenReturn(true)
+
+    val loggingEventCaptor = ArgumentCaptor.forClass(classOf[LogEvent])
 
     // Make sure only logging errors
-    val logger = Logger.getRootLogger
+    val logger = LogManager.getRootLogger().asInstanceOf[Logger]
     val oldLogLevel = logger.getLevel
     logger.setLevel(Level.ERROR)
     try {
@@ -331,14 +339,15 @@ class FileAppenderSuite extends SparkFunSuite with BeforeAndAfter with Logging {
       appender.awaitTermination()
 
       // Make sure no IOException errors have been logged as a result of appender closing gracefully
-      verify(mockAppender, atLeast(0)).doAppend(loggingEventCaptor.capture)
+      verify(mockAppender, atLeast(0)).append(loggingEventCaptor.capture)
       import scala.collection.JavaConverters._
       loggingEventCaptor.getAllValues.asScala.foreach { loggingEvent =>
-        assert(loggingEvent.getThrowableInformation === null
-          || !loggingEvent.getThrowableInformation.getThrowable.isInstanceOf[IOException])
+        assert(loggingEvent.getThrown === null
+          || !loggingEvent.getThrown.isInstanceOf[IOException])
       }
     } finally {
       logger.setLevel(oldLogLevel)
+      logger.removeAppender(mockAppender)
     }
   }
 
