@@ -36,6 +36,7 @@ import org.apache.commons.lang3.{JavaVersion, SystemUtils}
 import org.apache.commons.math3.stat.inference.ChiSquareTest
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.fs.Path
+import org.apache.logging.log4j.Level
 
 import org.apache.spark.{SparkConf, SparkException, SparkFunSuite, TaskContext}
 import org.apache.spark.internal.Logging
@@ -226,15 +227,16 @@ class UtilsSuite extends SparkFunSuite with ResetSystemProperties with Logging {
       try {
         // Get a handle on the buffered data, to make sure memory gets freed once we read past the
         // end of it. Need to use reflection to get handle on inner structures for this check
-        val byteBufferInputStream = if (mergedStream.isInstanceOf[ChunkedByteBufferInputStream]) {
-          assert(inputLength < limit)
-          mergedStream.asInstanceOf[ChunkedByteBufferInputStream]
-        } else {
-          assert(inputLength >= limit)
-          val sequenceStream = mergedStream.asInstanceOf[SequenceInputStream]
-          val fieldValue = getFieldValue(sequenceStream, "in")
-          assert(fieldValue.isInstanceOf[ChunkedByteBufferInputStream])
-          fieldValue.asInstanceOf[ChunkedByteBufferInputStream]
+        val byteBufferInputStream = mergedStream match {
+          case stream: ChunkedByteBufferInputStream =>
+            assert(inputLength < limit)
+            stream
+          case _ =>
+            assert(inputLength >= limit)
+            val sequenceStream = mergedStream.asInstanceOf[SequenceInputStream]
+            val fieldValue = getFieldValue(sequenceStream, "in")
+            assert(fieldValue.isInstanceOf[ChunkedByteBufferInputStream])
+            fieldValue.asInstanceOf[ChunkedByteBufferInputStream]
         }
         (0 until inputLength).foreach { idx =>
           assert(bytes(idx) === mergedStream.read().asInstanceOf[Byte])
@@ -462,7 +464,7 @@ class UtilsSuite extends SparkFunSuite with ResetSystemProperties with Logging {
 
   test("get iterator size") {
     val empty = Seq[Int]()
-    assert(Utils.getIteratorSize(empty.toIterator) === 0L)
+    assert(Utils.getIteratorSize(empty.iterator) === 0L)
     val iterator = Iterator.range(0, 5)
     assert(Utils.getIteratorSize(iterator) === 5L)
   }
@@ -477,6 +479,68 @@ class UtilsSuite extends SparkFunSuite with ResetSystemProperties with Logging {
     }
   }
 
+  test("SPARK-35907: createDirectory") {
+    val tmpDir = new File(System.getProperty("java.io.tmpdir"))
+    val testDir = new File(tmpDir, "createDirectory" + System.nanoTime())
+    val testDirPath = testDir.getCanonicalPath
+
+    // 1. Directory created successfully
+    val scenario1 = new File(testDir, "scenario1")
+    assert(Utils.createDirectory(scenario1))
+    assert(scenario1.exists())
+    assert(Utils.createDirectory(testDirPath, "scenario1").exists())
+
+    // 2. Illegal file path
+    val scenario2 = new File(testDir, "scenario2" * 256)
+    assert(!Utils.createDirectory(scenario2))
+    assert(!scenario2.exists())
+    assertThrows[IOException](Utils.createDirectory(testDirPath, "scenario2" * 256))
+
+    // 3. The parent directory cannot read
+    val scenario3 = new File(testDir, "scenario3")
+    assert(testDir.canRead)
+    assert(testDir.setReadable(false))
+    assert(Utils.createDirectory(scenario3))
+    assert(scenario3.exists())
+    assert(Utils.createDirectory(testDirPath, "scenario3").exists())
+    assert(testDir.setReadable(true))
+
+    // 4. The parent directory cannot write
+    val scenario4 = new File(testDir, "scenario4")
+    assert(testDir.canWrite)
+    assert(testDir.setWritable(false))
+    assert(!Utils.createDirectory(scenario4))
+    assert(!scenario4.exists())
+    assertThrows[IOException](Utils.createDirectory(testDirPath, "scenario4"))
+    assert(testDir.setWritable(true))
+
+    // 5. The parent directory cannot execute
+    val scenario5 = new File(testDir, "scenario5")
+    assert(testDir.canExecute)
+    assert(testDir.setExecutable(false))
+    assert(!Utils.createDirectory(scenario5))
+    assert(!scenario5.exists())
+    assertThrows[IOException](Utils.createDirectory(testDirPath, "scenario5"))
+    assert(testDir.setExecutable(true))
+
+    // The following 3 scenarios are only for the method: createDirectory(File)
+    // 6. Symbolic link
+    val scenario6 = java.nio.file.Files.createSymbolicLink(new File(testDir, "scenario6")
+      .toPath, scenario1.toPath).toFile
+    assert(!Utils.createDirectory(scenario6))
+    assert(scenario6.exists())
+
+    // 7. Directory exists
+    assert(scenario1.exists())
+    assert(Utils.createDirectory(scenario1))
+    assert(scenario1.exists())
+
+    // 8. Not directory
+    val scenario8 = new File(testDir.getCanonicalPath + File.separator + "scenario8")
+    assert(scenario8.createNewFile())
+    assert(!Utils.createDirectory(scenario8))
+  }
+
   test("doesDirectoryContainFilesNewerThan") {
     // create some temporary directories and files
     withTempDir { parent =>
@@ -623,11 +687,14 @@ class UtilsSuite extends SparkFunSuite with ResetSystemProperties with Logging {
 
   // Test for using the util function to change our log levels.
   test("log4j log level change") {
-    val current = org.apache.log4j.Logger.getRootLogger().getLevel()
+    val rootLogger = org.apache.logging.log4j.LogManager.getRootLogger()
+    val current = rootLogger.getLevel()
     try {
-      Utils.setLogLevel(org.apache.log4j.Level.ALL)
+      Utils.setLogLevel(Level.ALL)
+      assert(rootLogger.getLevel == Level.ALL)
       assert(log.isInfoEnabled())
-      Utils.setLogLevel(org.apache.log4j.Level.ERROR)
+      Utils.setLogLevel(Level.ERROR)
+      assert(rootLogger.getLevel == Level.ERROR)
       assert(!log.isInfoEnabled())
       assert(log.isErrorEnabled())
     } finally {
