@@ -45,12 +45,14 @@ def post_message_to_github(msg, ghprb_pull_id):
     github_oauth_key = os.environ["GITHUB_OAUTH_KEY"]
 
     posted_message = json.dumps({"body": msg})
-    request = Request(url,
-                      headers={
-                          "Authorization": "token %s" % github_oauth_key,
-                          "Content-Type": "application/json"
-                      },
-                      data=posted_message.encode('utf-8'))
+    request = Request(
+        url,
+        headers={
+            "Authorization": "token %s" % github_oauth_key,
+            "Content-Type": "application/json",
+        },
+        data=posted_message.encode("utf-8"),
+    )
     try:
         response = urlopen(request)
 
@@ -67,22 +69,20 @@ def post_message_to_github(msg, ghprb_pull_id):
         print_err(" > data: %s" % posted_message)
 
 
-def pr_message(build_display_name,
-               build_url,
-               ghprb_pull_id,
-               short_commit_hash,
-               commit_url,
-               msg,
-               post_msg=''):
+def pr_message(
+    build_display_name, build_url, ghprb_pull_id, short_commit_hash, commit_url, msg, post_msg=""
+):
     # align the arguments properly for string formatting
-    str_args = (build_display_name,
-                msg,
-                build_url,
-                ghprb_pull_id,
-                short_commit_hash,
-                commit_url,
-                str(' ' + post_msg + '.') if post_msg else '.')
-    return '**[Test build %s %s](%stestReport)** for PR %s at commit [`%s`](%s)%s' % str_args
+    str_args = (
+        build_display_name,
+        msg,
+        build_url,
+        ghprb_pull_id,
+        short_commit_hash,
+        commit_url,
+        str(" " + post_msg + ".") if post_msg else ".",
+    )
+    return "**[Test build %s %s](%stestReport)** for PR %s at commit [`%s`](%s)%s" % str_args
 
 
 def run_pr_checks(pr_tests, ghprb_actual_commit, sha1):
@@ -92,16 +92,24 @@ def run_pr_checks(pr_tests, ghprb_actual_commit, sha1):
     @return a list of messages to post back to GitHub
     """
     # Ensure we save off the current HEAD to revert to
-    current_pr_head = run_cmd(['git', 'rev-parse', 'HEAD'], return_output=True).strip()
+    current_pr_head = run_cmd(["git", "rev-parse", "HEAD"], return_output=True).strip()
     pr_results = list()
 
     for pr_test in pr_tests:
-        test_name = pr_test + '.sh'
-        pr_results.append(run_cmd(['bash', os.path.join(SPARK_HOME, 'dev', 'tests', test_name),
-                                   ghprb_actual_commit, sha1],
-                                  return_output=True).rstrip())
+        test_name = pr_test + ".sh"
+        pr_results.append(
+            run_cmd(
+                [
+                    "bash",
+                    os.path.join(SPARK_HOME, "dev", "tests", test_name),
+                    ghprb_actual_commit,
+                    sha1,
+                ],
+                return_output=True,
+            ).rstrip()
+        )
         # Ensure, after each test, that we're back on the current PR
-        run_cmd(['git', 'checkout', '-f', current_pr_head])
+        run_cmd(["git", "checkout", "-f", current_pr_head])
     return pr_results
 
 
@@ -112,37 +120,38 @@ def run_tests(tests_timeout):
     @return a tuple containing the test result code and the result note to post to GitHub
     """
 
-    test_result_code = subprocess.Popen(['timeout',
-                                         tests_timeout,
-                                         os.path.join(SPARK_HOME, 'dev', 'run-tests')]).wait()
+    test_result_code = subprocess.Popen(
+        ["timeout", tests_timeout, os.path.join(SPARK_HOME, "dev", "run-tests")]
+    ).wait()
 
     failure_note_by_errcode = {
         # error to denote run-tests script failures:
-        1: 'executing the `dev/run-tests` script',
-        ERROR_CODES["BLOCK_GENERAL"]: 'some tests',
-        ERROR_CODES["BLOCK_RAT"]: 'RAT tests',
-        ERROR_CODES["BLOCK_SCALA_STYLE"]: 'Scala style tests',
-        ERROR_CODES["BLOCK_JAVA_STYLE"]: 'Java style tests',
-        ERROR_CODES["BLOCK_PYTHON_STYLE"]: 'Python style tests',
-        ERROR_CODES["BLOCK_R_STYLE"]: 'R style tests',
-        ERROR_CODES["BLOCK_DOCUMENTATION"]: 'to generate documentation',
-        ERROR_CODES["BLOCK_BUILD"]: 'to build',
-        ERROR_CODES["BLOCK_BUILD_TESTS"]: 'build dependency tests',
-        ERROR_CODES["BLOCK_MIMA"]: 'MiMa tests',
-        ERROR_CODES["BLOCK_SPARK_UNIT_TESTS"]: 'Spark unit tests',
-        ERROR_CODES["BLOCK_PYSPARK_UNIT_TESTS"]: 'PySpark unit tests',
-        ERROR_CODES["BLOCK_PYSPARK_PIP_TESTS"]: 'PySpark pip packaging tests',
-        ERROR_CODES["BLOCK_SPARKR_UNIT_TESTS"]: 'SparkR unit tests',
-        ERROR_CODES["BLOCK_TIMEOUT"]: 'from timeout after a configured wait of `%s`' % (
-            tests_timeout)
+        1: "executing the `dev/run-tests` script",
+        ERROR_CODES["BLOCK_GENERAL"]: "some tests",
+        ERROR_CODES["BLOCK_RAT"]: "RAT tests",
+        ERROR_CODES["BLOCK_SCALA_STYLE"]: "Scala style tests",
+        ERROR_CODES["BLOCK_JAVA_STYLE"]: "Java style tests",
+        ERROR_CODES["BLOCK_PYTHON_STYLE"]: "Python style tests",
+        ERROR_CODES["BLOCK_R_STYLE"]: "R style tests",
+        ERROR_CODES["BLOCK_DOCUMENTATION"]: "to generate documentation",
+        ERROR_CODES["BLOCK_BUILD"]: "to build",
+        ERROR_CODES["BLOCK_BUILD_TESTS"]: "build dependency tests",
+        ERROR_CODES["BLOCK_MIMA"]: "MiMa tests",
+        ERROR_CODES["BLOCK_SPARK_UNIT_TESTS"]: "Spark unit tests",
+        ERROR_CODES["BLOCK_PYSPARK_UNIT_TESTS"]: "PySpark unit tests",
+        ERROR_CODES["BLOCK_PYSPARK_PIP_TESTS"]: "PySpark pip packaging tests",
+        ERROR_CODES["BLOCK_SPARKR_UNIT_TESTS"]: "SparkR unit tests",
+        ERROR_CODES["BLOCK_TIMEOUT"]: "from timeout after a configured wait of `%s`"
+        % (tests_timeout),
     }
 
     if test_result_code == 0:
-        test_result_note = ' * This patch passes all tests.'
+        test_result_note = " * This patch passes all tests."
     else:
         note = failure_note_by_errcode.get(
-            test_result_code, "due to an unknown error code, %s" % test_result_code)
-        test_result_note = ' * This patch **fails %s**.' % note
+            test_result_code, "due to an unknown error code, %s" % test_result_code
+        )
+        test_result_note = " * This patch **fails %s**." % note
 
     return [test_result_code, test_result_note]
 
@@ -170,13 +179,10 @@ def main():
     if "test-maven" in ghprb_pull_title:
         os.environ["AMPLAB_JENKINS_BUILD_TOOL"] = "maven"
     # Switch the Hadoop profile based on the PR title:
-    if "test-hadoop2.7" in ghprb_pull_title:
-        os.environ["AMPLAB_JENKINS_BUILD_PROFILE"] = "hadoop2.7"
-    if "test-hadoop3.2" in ghprb_pull_title:
-        os.environ["AMPLAB_JENKINS_BUILD_PROFILE"] = "hadoop3.2"
-    # Switch the Hive profile based on the PR title:
-    if "test-hive2.3" in ghprb_pull_title:
-        os.environ["AMPLAB_JENKINS_BUILD_HIVE_PROFILE"] = "hive2.3"
+    if "test-hadoop2" in ghprb_pull_title:
+        os.environ["AMPLAB_JENKINS_BUILD_PROFILE"] = "hadoop2"
+    if "test-hadoop3" in ghprb_pull_title:
+        os.environ["AMPLAB_JENKINS_BUILD_PROFILE"] = "hadoop3"
     # Switch the Scala profile based on the PR title:
     if "test-scala2.13" in ghprb_pull_title:
         os.environ["AMPLAB_JENKINS_BUILD_SCALA_PROFILE"] = "scala2.13"
@@ -205,30 +211,24 @@ def main():
     #     hash, the second the GitHub SHA1 hash, and the final the current PR hash
     #   * and, lastly, return string output to be included in the pr message output that will
     #     be posted to GitHub
-    pr_tests = [
-        "pr_merge_ability",
-        "pr_public_classes"
-    ]
+    pr_tests = ["pr_merge_ability", "pr_public_classes"]
 
     # `bind_message_base` returns a function to generate messages for GitHub posting
-    github_message = functools.partial(pr_message,
-                                       build_display_name,
-                                       build_url,
-                                       ghprb_pull_id,
-                                       short_commit_hash,
-                                       commit_url)
+    github_message = functools.partial(
+        pr_message, build_display_name, build_url, ghprb_pull_id, short_commit_hash, commit_url
+    )
 
     # post start message
-    post_message_to_github(github_message('has started'), ghprb_pull_id)
+    post_message_to_github(github_message("has started"), ghprb_pull_id)
 
     pr_check_results = run_pr_checks(pr_tests, ghprb_actual_commit, sha1)
 
     test_result_code, test_result_note = run_tests(tests_timeout)
 
     # post end message
-    result_message = github_message('has finished')
-    result_message += '\n' + test_result_note + '\n'
-    result_message += '\n'.join(pr_check_results)
+    result_message = github_message("has finished")
+    result_message += "\n" + test_result_note + "\n"
+    result_message += "\n".join(pr_check_results)
 
     post_message_to_github(result_message, ghprb_pull_id)
 
