@@ -86,13 +86,13 @@ object MimaBuild {
 
   def mimaSettings(sparkHome: File, projectRef: ProjectRef): Seq[Setting[_]] = {
     val organization = "org.apache.spark"
-    val previousSparkVersion = "3.0.0"
+    val previousSparkVersion = "3.2.0"
     val project = projectRef.project
-    val fullId = "spark-" + project + "_2.12"
+    val id = "spark-" + project
 
     Seq(
       mimaFailOnNoPrevious := true,
-      mimaPreviousArtifacts := Set(organization % fullId % previousSparkVersion),
+      mimaPreviousArtifacts := Set(organization %% id % previousSparkVersion),
       mimaBinaryIssueFilters ++= ignoredABIProblems(sparkHome, version.value)
     )
   }
