@@ -19,23 +19,59 @@ import sys
 import warnings
 from functools import reduce
 from threading import RLock
+from types import TracebackType
+from typing import (
+    Any,
+    ClassVar,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    cast,
+    no_type_check,
+    overload,
+    TYPE_CHECKING,
+)
 
-from pyspark import since
+from py4j.java_gateway import JavaObject
+
+from pyspark import SparkConf, SparkContext, since
 from pyspark.rdd import RDD
 from pyspark.sql.conf import RuntimeConfig
 from pyspark.sql.dataframe import DataFrame
 from pyspark.sql.pandas.conversion import SparkConversionMixin
 from pyspark.sql.readwriter import DataFrameReader
+from pyspark.sql.sql_formatter import SQLStringFormatter
 from pyspark.sql.streaming import DataStreamReader
-from pyspark.sql.types import DataType, StructType, \
-    _make_type_verifier, _infer_schema, _has_nulltype, _merge_type, _create_converter, \
-    _parse_datatype_string
-from pyspark.sql.utils import install_exception_handler
+from pyspark.sql.types import (
+    AtomicType,
+    DataType,
+    StructType,
+    _make_type_verifier,
+    _infer_schema,
+    _has_nulltype,
+    _merge_type,
+    _create_converter,
+    _parse_datatype_string,
+)
+from pyspark.sql.utils import install_exception_handler, is_timestamp_ntz_preferred
+
+if TYPE_CHECKING:
+    from pyspark.sql._typing import AtomicValue, RowLike
+    from pyspark.sql.catalog import Catalog
+    from pyspark.sql.pandas._typing import DataFrameLike as PandasDataFrameLike
+    from pyspark.sql.streaming import StreamingQueryManager
+    from pyspark.sql.udf import UDFRegistration
+
 
 __all__ = ["SparkSession"]
 
 
-def _monkey_patch_RDD(sparkSession):
+def _monkey_patch_RDD(sparkSession: "SparkSession") -> None:
+    @no_type_check
     def toDF(self, schema=None, sampleRatio=None):
         """
         Converts current :class:`RDD` into a :class:`DataFrame`
@@ -65,7 +101,7 @@ def _monkey_patch_RDD(sparkSession):
         """
         return sparkSession.createDataFrame(self, schema, sampleRatio)
 
-    RDD.toDF = toDF
+    RDD.toDF = toDF  # type: ignore[assignment]
 
 
 class SparkSession(SparkConversionMixin):
@@ -102,15 +138,27 @@ class SparkSession(SparkConversionMixin):
     [(1, 'string', 1.0, 1, True, datetime.datetime(2014, 8, 1, 14, 1, 5), 1, [1, 2, 3])]
     """
 
-    class Builder(object):
-        """Builder for :class:`SparkSession`.
-        """
+    class Builder:
+        """Builder for :class:`SparkSession`."""
 
         _lock = RLock()
-        _options = {}
-        _sc = None
+        _options: Dict[str, Any] = {}
+        _sc: Optional[SparkContext] = None
 
-        def config(self, key=None, value=None, conf=None):
+        @overload
+        def config(self, *, conf: SparkConf) -> "SparkSession.Builder":
+            ...
+
+        @overload
+        def config(self, key: str, value: Any) -> "SparkSession.Builder":
+            ...
+
+        def config(
+            self,
+            key: Optional[str] = None,
+            value: Optional[Any] = None,
+            conf: Optional[SparkConf] = None,
+        ) -> "SparkSession.Builder":
             """Sets a config option. Options set using this method are automatically propagated to
             both :class:`SparkConf` and :class:`SparkSession`'s own configuration.
 
@@ -141,13 +189,13 @@ class SparkSession(SparkConversionMixin):
             """
             with self._lock:
                 if conf is None:
-                    self._options[key] = str(value)
+                    self._options[cast(str, key)] = str(value)
                 else:
                     for (k, v) in conf.getAll():
                         self._options[k] = v
                 return self
 
-        def master(self, master):
+        def master(self, master: str) -> "SparkSession.Builder":
             """Sets the Spark master URL to connect to, such as "local" to run locally, "local[4]"
             to run locally with 4 cores, or "spark://master:7077" to run on a Spark standalone
             cluster.
@@ -161,7 +209,7 @@ class SparkSession(SparkConversionMixin):
             """
             return self.config("spark.master", master)
 
-        def appName(self, name):
+        def appName(self, name: str) -> "SparkSession.Builder":
             """Sets a name for the application, which will be shown in the Spark web UI.
 
             If no application name is set, a randomly generated name will be used.
@@ -176,18 +224,13 @@ class SparkSession(SparkConversionMixin):
             return self.config("spark.app.name", name)
 
         @since(2.0)
-        def enableHiveSupport(self):
+        def enableHiveSupport(self) -> "SparkSession.Builder":
             """Enables Hive support, including connectivity to a persistent Hive metastore, support
             for Hive SerDes, and Hive user-defined functions.
             """
             return self.config("spark.sql.catalogImplementation", "hive")
 
-        def _sparkContext(self, sc):
-            with self._lock:
-                self._sc = sc
-                return self
-
-        def getOrCreate(self):
+        def getOrCreate(self) -> "SparkSession":
             """Gets an existing :class:`SparkSession` or, if there is no existing one, creates a
             new one based on the options set in this builder.
 
@@ -216,57 +259,73 @@ class SparkSession(SparkConversionMixin):
             with self._lock:
                 from pyspark.context import SparkContext
                 from pyspark.conf import SparkConf
+
                 session = SparkSession._instantiatedSession
                 if session is None or session._sc._jsc is None:
-                    if self._sc is not None:
-                        sc = self._sc
-                    else:
-                        sparkConf = SparkConf()
-                        for key, value in self._options.items():
-                            sparkConf.set(key, value)
-                        # This SparkContext may be an existing one.
-                        sc = SparkContext.getOrCreate(sparkConf)
+                    sparkConf = SparkConf()
+                    for key, value in self._options.items():
+                        sparkConf.set(key, value)
+                    # This SparkContext may be an existing one.
+                    sc = SparkContext.getOrCreate(sparkConf)
                     # Do not update `SparkConf` for existing `SparkContext`, as it's shared
                     # by all sessions.
-                    session = SparkSession(sc)
-                for key, value in self._options.items():
-                    session._jsparkSession.sessionState().conf().setConfString(key, value)
+                    session = SparkSession(sc, options=self._options)
+                else:
+                    getattr(
+                        getattr(session._jvm, "SparkSession$"), "MODULE$"
+                    ).applyModifiableSettings(session._jsparkSession, self._options)
                 return session
 
     builder = Builder()
     """A class attribute having a :class:`Builder` to construct :class:`SparkSession` instances."""
 
-    _instantiatedSession = None
-    _activeSession = None
+    _instantiatedSession: ClassVar[Optional["SparkSession"]] = None
+    _activeSession: ClassVar[Optional["SparkSession"]] = None
 
-    def __init__(self, sparkContext, jsparkSession=None):
-        from pyspark.sql.context import SQLContext
+    def __init__(
+        self,
+        sparkContext: SparkContext,
+        jsparkSession: Optional[JavaObject] = None,
+        options: Dict[str, Any] = {},
+    ):
         self._sc = sparkContext
         self._jsc = self._sc._jsc
         self._jvm = self._sc._jvm
+
+        assert self._jvm is not None
+
         if jsparkSession is None:
-            if self._jvm.SparkSession.getDefaultSession().isDefined() \
-                    and not self._jvm.SparkSession.getDefaultSession().get() \
-                        .sparkContext().isStopped():
+            if (
+                self._jvm.SparkSession.getDefaultSession().isDefined()
+                and not self._jvm.SparkSession.getDefaultSession().get().sparkContext().isStopped()
+            ):
                 jsparkSession = self._jvm.SparkSession.getDefaultSession().get()
+                getattr(getattr(self._jvm, "SparkSession$"), "MODULE$").applyModifiableSettings(
+                    jsparkSession, options
+                )
             else:
-                jsparkSession = self._jvm.SparkSession(self._jsc.sc())
+                jsparkSession = self._jvm.SparkSession(self._jsc.sc(), options)
+        else:
+            getattr(getattr(self._jvm, "SparkSession$"), "MODULE$").applyModifiableSettings(
+                jsparkSession, options
+            )
         self._jsparkSession = jsparkSession
-        self._jwrapped = self._jsparkSession.sqlContext()
-        self._wrapped = SQLContext(self._sc, self, self._jwrapped)
         _monkey_patch_RDD(self)
         install_exception_handler()
         # If we had an instantiated SparkSession attached with a SparkContext
         # which is stopped now, we need to renew the instantiated SparkSession.
         # Otherwise, we will use invalid SparkSession when we call Builder.getOrCreate.
-        if SparkSession._instantiatedSession is None \
-                or SparkSession._instantiatedSession._sc._jsc is None:
+        if (
+            SparkSession._instantiatedSession is None
+            or SparkSession._instantiatedSession._sc._jsc is None
+        ):
             SparkSession._instantiatedSession = self
             SparkSession._activeSession = self
+            assert self._jvm is not None
             self._jvm.SparkSession.setDefaultSession(self._jsparkSession)
             self._jvm.SparkSession.setActiveSession(self._jsparkSession)
 
-    def _repr_html_(self):
+    def _repr_html_(self) -> str:
         return """
             <div>
                 <p><b>SparkSession - {catalogImplementation}</b></p>
@@ -274,11 +333,16 @@ class SparkSession(SparkConversionMixin):
             </div>
         """.format(
             catalogImplementation=self.conf.get("spark.sql.catalogImplementation"),
-            sc_HTML=self.sparkContext._repr_html_()
+            sc_HTML=self.sparkContext._repr_html_(),
         )
 
+    @property
+    def _jconf(self) -> "JavaObject":
+        """Accessor for the JVM SQL-specific configurations"""
+        return self._jsparkSession.sessionState().conf()
+
     @since(2.0)
-    def newSession(self):
+    def newSession(self) -> "SparkSession":
         """
         Returns a new :class:`SparkSession` as new session, that has separate SQLConf,
         registered temporary views and UDFs, but shared :class:`SparkContext` and
@@ -287,7 +351,7 @@ class SparkSession(SparkConversionMixin):
         return self.__class__(self._sc, self._jsparkSession.newSession())
 
     @classmethod
-    def getActiveSession(cls):
+    def getActiveSession(cls) -> Optional["SparkSession"]:
         """
         Returns the active :class:`SparkSession` for the current thread, returned by the builder
 
@@ -308,31 +372,33 @@ class SparkSession(SparkConversionMixin):
         [Row(age=1)]
         """
         from pyspark import SparkContext
+
         sc = SparkContext._active_spark_context
         if sc is None:
             return None
         else:
+            assert sc._jvm is not None
             if sc._jvm.SparkSession.getActiveSession().isDefined():
                 SparkSession(sc, sc._jvm.SparkSession.getActiveSession().get())
                 return SparkSession._activeSession
             else:
                 return None
 
-    @property
+    @property  # type: ignore[misc]
     @since(2.0)
-    def sparkContext(self):
+    def sparkContext(self) -> SparkContext:
         """Returns the underlying :class:`SparkContext`."""
         return self._sc
 
-    @property
+    @property  # type: ignore[misc]
     @since(2.0)
-    def version(self):
+    def version(self) -> str:
         """The version of Spark on which this application is running."""
         return self._jsparkSession.version()
 
-    @property
+    @property  # type: ignore[misc]
     @since(2.0)
-    def conf(self):
+    def conf(self) -> RuntimeConfig:
         """Runtime configuration interface for Spark.
 
         This is the interface through which the user can get and set all Spark and Hadoop
@@ -348,7 +414,7 @@ class SparkSession(SparkConversionMixin):
         return self._conf
 
     @property
-    def catalog(self):
+    def catalog(self) -> "Catalog":
         """Interface through which the user may create, drop, alter or query underlying
         databases, tables, functions, etc.
 
@@ -359,12 +425,13 @@ class SparkSession(SparkConversionMixin):
         :class:`Catalog`
         """
         from pyspark.sql.catalog import Catalog
+
         if not hasattr(self, "_catalog"):
             self._catalog = Catalog(self)
         return self._catalog
 
     @property
-    def udf(self):
+    def udf(self) -> "UDFRegistration":
         """Returns a :class:`UDFRegistration` for UDF registration.
 
         .. versionadded:: 2.0.0
@@ -374,9 +441,16 @@ class SparkSession(SparkConversionMixin):
         :class:`UDFRegistration`
         """
         from pyspark.sql.udf import UDFRegistration
+
         return UDFRegistration(self)
 
-    def range(self, start, end=None, step=1, numPartitions=None):
+    def range(
+        self,
+        start: int,
+        end: Optional[int] = None,
+        step: int = 1,
+        numPartitions: Optional[int] = None,
+    ) -> DataFrame:
         """
         Create a :class:`DataFrame` with single :class:`pyspark.sql.types.LongType` column named
         ``id``, containing elements in a range from ``start`` to ``end`` (exclusive) with
@@ -417,9 +491,11 @@ class SparkSession(SparkConversionMixin):
         else:
             jdf = self._jsparkSession.range(int(start), int(end), int(step), int(numPartitions))
 
-        return DataFrame(jdf, self._wrapped)
+        return DataFrame(jdf, self)
 
-    def _inferSchemaFromList(self, data, names=None):
+    def _inferSchemaFromList(
+        self, data: Iterable[Any], names: Optional[List[str]] = None
+    ) -> StructType:
         """
         Infer schema from list of Row, dict, or tuple.
 
@@ -436,12 +512,22 @@ class SparkSession(SparkConversionMixin):
         """
         if not data:
             raise ValueError("can not infer schema from empty dataset")
-        schema = reduce(_merge_type, (_infer_schema(row, names) for row in data))
+        infer_dict_as_struct = self._jconf.inferDictAsStruct()
+        prefer_timestamp_ntz = is_timestamp_ntz_preferred()
+        schema = reduce(
+            _merge_type,
+            (_infer_schema(row, names, infer_dict_as_struct, prefer_timestamp_ntz) for row in data),
+        )
         if _has_nulltype(schema):
             raise ValueError("Some of types cannot be determined after inferring")
         return schema
 
-    def _inferSchema(self, rdd, samplingRatio=None, names=None):
+    def _inferSchema(
+        self,
+        rdd: RDD[Any],
+        samplingRatio: Optional[float] = None,
+        names: Optional[List[str]] = None,
+    ) -> StructType:
         """
         Infer schema from an RDD of Row, dict, or tuple.
 
@@ -459,47 +545,80 @@ class SparkSession(SparkConversionMixin):
         """
         first = rdd.first()
         if not first:
-            raise ValueError("The first row in RDD is empty, "
-                             "can not infer schema")
+            raise ValueError("The first row in RDD is empty, " "can not infer schema")
 
+        infer_dict_as_struct = self._jconf.inferDictAsStruct()
+        prefer_timestamp_ntz = is_timestamp_ntz_preferred()
         if samplingRatio is None:
-            schema = _infer_schema(first, names=names)
+            schema = _infer_schema(
+                first,
+                names=names,
+                infer_dict_as_struct=infer_dict_as_struct,
+                prefer_timestamp_ntz=prefer_timestamp_ntz,
+            )
             if _has_nulltype(schema):
                 for row in rdd.take(100)[1:]:
-                    schema = _merge_type(schema, _infer_schema(row, names=names))
+                    schema = _merge_type(
+                        schema,
+                        _infer_schema(
+                            row,
+                            names=names,
+                            infer_dict_as_struct=infer_dict_as_struct,
+                            prefer_timestamp_ntz=prefer_timestamp_ntz,
+                        ),
+                    )
                     if not _has_nulltype(schema):
                         break
                 else:
-                    raise ValueError("Some of types cannot be determined by the "
-                                     "first 100 rows, please try again with sampling")
+                    raise ValueError(
+                        "Some of types cannot be determined by the "
+                        "first 100 rows, please try again with sampling"
+                    )
         else:
             if samplingRatio < 0.99:
                 rdd = rdd.sample(False, float(samplingRatio))
-            schema = rdd.map(lambda row: _infer_schema(row, names)).reduce(_merge_type)
+            schema = rdd.map(
+                lambda row: _infer_schema(
+                    row,
+                    names,
+                    infer_dict_as_struct=infer_dict_as_struct,
+                    prefer_timestamp_ntz=prefer_timestamp_ntz,
+                )
+            ).reduce(_merge_type)
         return schema
 
-    def _createFromRDD(self, rdd, schema, samplingRatio):
+    def _createFromRDD(
+        self,
+        rdd: RDD[Any],
+        schema: Optional[Union[DataType, List[str]]],
+        samplingRatio: Optional[float],
+    ) -> Tuple[RDD[Tuple], StructType]:
         """
         Create an RDD for DataFrame from an existing RDD, returns the RDD and schema.
         """
         if schema is None or isinstance(schema, (list, tuple)):
             struct = self._inferSchema(rdd, samplingRatio, names=schema)
             converter = _create_converter(struct)
-            rdd = rdd.map(converter)
+            tupled_rdd = rdd.map(converter)
             if isinstance(schema, (list, tuple)):
                 for i, name in enumerate(schema):
                     struct.fields[i].name = name
                     struct.names[i] = name
-            schema = struct
 
-        elif not isinstance(schema, StructType):
+        elif isinstance(schema, StructType):
+            struct = schema
+            tupled_rdd = rdd
+
+        else:
             raise TypeError("schema should be StructType or list or None, but got: %s" % schema)
 
         # convert python objects to sql data
-        rdd = rdd.map(schema.toInternal)
-        return rdd, schema
+        internal_rdd = tupled_rdd.map(struct.toInternal)
+        return internal_rdd, struct
 
-    def _createFromLocal(self, data, schema):
+    def _createFromLocal(
+        self, data: Iterable[Any], schema: Optional[Union[DataType, List[str]]]
+    ) -> Tuple[RDD[Tuple], StructType]:
         """
         Create an RDD for DataFrame from a list or pandas.DataFrame, returns
         the RDD and schema.
@@ -511,22 +630,25 @@ class SparkSession(SparkConversionMixin):
         if schema is None or isinstance(schema, (list, tuple)):
             struct = self._inferSchemaFromList(data, names=schema)
             converter = _create_converter(struct)
-            data = map(converter, data)
+            tupled_data: Iterable[Tuple] = map(converter, data)
             if isinstance(schema, (list, tuple)):
                 for i, name in enumerate(schema):
                     struct.fields[i].name = name
                     struct.names[i] = name
-            schema = struct
 
-        elif not isinstance(schema, StructType):
+        elif isinstance(schema, StructType):
+            struct = schema
+            tupled_data = data
+
+        else:
             raise TypeError("schema should be StructType or list or None, but got: %s" % schema)
 
         # convert python objects to sql data
-        data = [schema.toInternal(row) for row in data]
-        return self._sc.parallelize(data), schema
+        internal_data = [struct.toInternal(row) for row in tupled_data]
+        return self._sc.parallelize(internal_data), struct
 
     @staticmethod
-    def _create_shell_session():
+    def _create_shell_session() -> "SparkSession":
         """
         Initialize a :class:`SparkSession` for a pyspark shell session. This is called from
         shell.py to make error handling simpler without needing to declare local variables in
@@ -535,24 +657,120 @@ class SparkSession(SparkConversionMixin):
         import py4j
         from pyspark.conf import SparkConf
         from pyspark.context import SparkContext
+
         try:
             # Try to access HiveConf, it will raise exception if Hive is not added
             conf = SparkConf()
-            if conf.get('spark.sql.catalogImplementation', 'hive').lower() == 'hive':
+            assert SparkContext._jvm is not None
+            if conf.get("spark.sql.catalogImplementation", "hive").lower() == "hive":
                 SparkContext._jvm.org.apache.hadoop.hive.conf.HiveConf()
-                return SparkSession.builder\
-                    .enableHiveSupport()\
-                    .getOrCreate()
+                return SparkSession.builder.enableHiveSupport().getOrCreate()
             else:
-                return SparkSession.builder.getOrCreate()
+                return SparkSession._getActiveSessionOrCreate()
         except (py4j.protocol.Py4JError, TypeError):
-            if conf.get('spark.sql.catalogImplementation', '').lower() == 'hive':
-                warnings.warn("Fall back to non-hive support because failing to access HiveConf, "
-                              "please make sure you build spark with hive")
+            if conf.get("spark.sql.catalogImplementation", "").lower() == "hive":
+                warnings.warn(
+                    "Fall back to non-hive support because failing to access HiveConf, "
+                    "please make sure you build spark with hive"
+                )
 
-        return SparkSession.builder.getOrCreate()
+        return SparkSession._getActiveSessionOrCreate()
 
-    def createDataFrame(self, data, schema=None, samplingRatio=None, verifySchema=True):
+    @staticmethod
+    def _getActiveSessionOrCreate(**static_conf: Any) -> "SparkSession":
+        """
+        Returns the active :class:`SparkSession` for the current thread, returned by the builder,
+        or if there is no existing one, creates a new one based on the options set in the builder.
+
+        NOTE that 'static_conf' might not be set if there's an active or default Spark session
+        running.
+        """
+        spark = SparkSession.getActiveSession()
+        if spark is None:
+            builder = SparkSession.builder
+            for k, v in static_conf.items():
+                builder = builder.config(k, v)
+            spark = builder.getOrCreate()
+        return spark
+
+    @overload
+    def createDataFrame(
+        self,
+        data: Iterable["RowLike"],
+        schema: Union[List[str], Tuple[str, ...]] = ...,
+        samplingRatio: Optional[float] = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: "RDD[RowLike]",
+        schema: Union[List[str], Tuple[str, ...]] = ...,
+        samplingRatio: Optional[float] = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: Iterable["RowLike"],
+        schema: Union[StructType, str],
+        *,
+        verifySchema: bool = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: "RDD[RowLike]",
+        schema: Union[StructType, str],
+        *,
+        verifySchema: bool = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: "RDD[AtomicValue]",
+        schema: Union[AtomicType, str],
+        verifySchema: bool = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: Iterable["AtomicValue"],
+        schema: Union[AtomicType, str],
+        verifySchema: bool = ...,
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self, data: "PandasDataFrameLike", samplingRatio: Optional[float] = ...
+    ) -> DataFrame:
+        ...
+
+    @overload
+    def createDataFrame(
+        self,
+        data: "PandasDataFrameLike",
+        schema: Union[StructType, str],
+        verifySchema: bool = ...,
+    ) -> DataFrame:
+        ...
+
+    def createDataFrame(  # type: ignore[misc]
+        self,
+        data: Union[RDD[Any], Iterable[Any], "PandasDataFrameLike"],
+        schema: Optional[Union[AtomicType, StructType, str]] = None,
+        samplingRatio: Optional[float] = None,
+        verifySchema: bool = True,
+    ) -> DataFrame:
         """
         Creates a :class:`DataFrame` from an :class:`RDD`, a list or a :class:`pandas.DataFrame`.
 
@@ -587,9 +805,7 @@ class SparkSession(SparkConversionMixin):
             a :class:`pyspark.sql.types.DataType` or a datatype string or a list of
             column names, default is None.  The data type string format equals to
             :class:`pyspark.sql.types.DataType.simpleString`, except that top level struct type can
-            omit the ``struct<>`` and atomic types use ``typeName()`` as their format, e.g. use
-            ``byte`` instead of ``tinyint`` for :class:`pyspark.sql.types.ByteType`.
-            We can also use ``int`` as a short name for :class:`pyspark.sql.types.IntegerType`.
+            omit the ``struct<>``.
         samplingRatio : float, optional
             the sample ratio of rows used for inferring
         verifySchema : bool, optional
@@ -653,76 +869,174 @@ class SparkSession(SparkConversionMixin):
         Py4JJavaError: ...
         """
         SparkSession._activeSession = self
+        assert self._jvm is not None
         self._jvm.SparkSession.setActiveSession(self._jsparkSession)
         if isinstance(data, DataFrame):
             raise TypeError("data is already a DataFrame")
 
         if isinstance(schema, str):
-            schema = _parse_datatype_string(schema)
+            schema = cast(Union[AtomicType, StructType, str], _parse_datatype_string(schema))
         elif isinstance(schema, (list, tuple)):
             # Must re-encode any unicode strings to be consistent with StructField names
-            schema = [x.encode('utf-8') if not isinstance(x, str) else x for x in schema]
+            schema = [x.encode("utf-8") if not isinstance(x, str) else x for x in schema]
 
         try:
             import pandas
+
             has_pandas = True
         except Exception:
             has_pandas = False
         if has_pandas and isinstance(data, pandas.DataFrame):
             # Create a DataFrame from pandas DataFrame.
-            return super(SparkSession, self).createDataFrame(
-                data, schema, samplingRatio, verifySchema)
-        return self._create_dataframe(data, schema, samplingRatio, verifySchema)
+            return super(SparkSession, self).createDataFrame(  # type: ignore[call-overload]
+                data, schema, samplingRatio, verifySchema
+            )
+        return self._create_dataframe(
+            data, schema, samplingRatio, verifySchema  # type: ignore[arg-type]
+        )
 
-    def _create_dataframe(self, data, schema, samplingRatio, verifySchema):
+    def _create_dataframe(
+        self,
+        data: Union[RDD[Any], Iterable[Any]],
+        schema: Optional[Union[DataType, List[str]]],
+        samplingRatio: Optional[float],
+        verifySchema: bool,
+    ) -> DataFrame:
         if isinstance(schema, StructType):
             verify_func = _make_type_verifier(schema) if verifySchema else lambda _: True
 
+            @no_type_check
             def prepare(obj):
                 verify_func(obj)
                 return obj
+
         elif isinstance(schema, DataType):
             dataType = schema
             schema = StructType().add("value", schema)
 
-            verify_func = _make_type_verifier(
-                dataType, name="field value") if verifySchema else lambda _: True
+            verify_func = (
+                _make_type_verifier(dataType, name="field value")
+                if verifySchema
+                else lambda _: True
+            )
 
+            @no_type_check
             def prepare(obj):
                 verify_func(obj)
-                return obj,
+                return (obj,)
+
         else:
-            prepare = lambda obj: obj
+
+            def prepare(obj: Any) -> Any:
+                return obj
 
         if isinstance(data, RDD):
-            rdd, schema = self._createFromRDD(data.map(prepare), schema, samplingRatio)
+            rdd, struct = self._createFromRDD(data.map(prepare), schema, samplingRatio)
         else:
-            rdd, schema = self._createFromLocal(map(prepare, data), schema)
+            rdd, struct = self._createFromLocal(map(prepare, data), schema)
+        assert self._jvm is not None
         jrdd = self._jvm.SerDeUtil.toJavaArray(rdd._to_java_object_rdd())
-        jdf = self._jsparkSession.applySchemaToPythonRDD(jrdd.rdd(), schema.json())
-        df = DataFrame(jdf, self._wrapped)
-        df._schema = schema
+        jdf = self._jsparkSession.applySchemaToPythonRDD(jrdd.rdd(), struct.json())
+        df = DataFrame(jdf, self)
+        df._schema = struct
         return df
 
-    def sql(self, sqlQuery):
+    def sql(self, sqlQuery: str, **kwargs: Any) -> DataFrame:
         """Returns a :class:`DataFrame` representing the result of the given query.
+        When ``kwargs`` is specified, this method formats the given string by using the Python
+        standard formatter.
 
         .. versionadded:: 2.0.0
 
+        Parameters
+        ----------
+        sqlQuery : str
+            SQL query string.
+        kwargs : dict
+            Other variables that the user wants to set that can be referenced in the query
+
+            .. versionchanged:: 3.3.0
+               Added optional argument ``kwargs`` to specify the mapping of variables in the query.
+               This feature is experimental and unstable.
+
         Returns
         -------
         :class:`DataFrame`
 
         Examples
         --------
-        >>> df.createOrReplaceTempView("table1")
-        >>> df2 = spark.sql("SELECT field1 AS f1, field2 as f2 from table1")
-        >>> df2.collect()
-        [Row(f1=1, f2='row1'), Row(f1=2, f2='row2'), Row(f1=3, f2='row3')]
-        """
-        return DataFrame(self._jsparkSession.sql(sqlQuery), self._wrapped)
+        Executing a SQL query.
 
-    def table(self, tableName):
+        >>> spark.sql("SELECT * FROM range(10) where id > 7").show()
+        +---+
+        | id|
+        +---+
+        |  8|
+        |  9|
+        +---+
+
+        Executing a SQL query with variables as Python formatter standard.
+
+        >>> spark.sql(
+        ...     "SELECT * FROM range(10) WHERE id > {bound1} AND id < {bound2}", bound1=7, bound2=9
+        ... ).show()
+        +---+
+        | id|
+        +---+
+        |  8|
+        +---+
+
+        >>> mydf = spark.range(10)
+        >>> spark.sql(
+        ...     "SELECT {col} FROM {mydf} WHERE id IN {x}",
+        ...     col=mydf.id, mydf=mydf, x=tuple(range(4))).show()
+        +---+
+        | id|
+        +---+
+        |  0|
+        |  1|
+        |  2|
+        |  3|
+        +---+
+
+        >>> spark.sql('''
+        ...   SELECT m1.a, m2.b
+        ...   FROM {table1} m1 INNER JOIN {table2} m2
+        ...   ON m1.key = m2.key
+        ...   ORDER BY m1.a, m2.b''',
+        ...   table1=spark.createDataFrame([(1, "a"), (2, "b")], ["a", "key"]),
+        ...   table2=spark.createDataFrame([(3, "a"), (4, "b"), (5, "b")], ["b", "key"])).show()
+        +---+---+
+        |  a|  b|
+        +---+---+
+        |  1|  3|
+        |  2|  4|
+        |  2|  5|
+        +---+---+
+
+        Also, it is possible to query using class:`Column` from :class:`DataFrame`.
+
+        >>> mydf = spark.createDataFrame([(1, 4), (2, 4), (3, 6)], ["A", "B"])
+        >>> spark.sql("SELECT {df.A}, {df[B]} FROM {df}", df=mydf).show()
+        +---+---+
+        |  A|  B|
+        +---+---+
+        |  1|  4|
+        |  2|  4|
+        |  3|  6|
+        +---+---+
+        """
+
+        formatter = SQLStringFormatter(self)
+        if len(kwargs) > 0:
+            sqlQuery = formatter.format(sqlQuery, **kwargs)
+        try:
+            return DataFrame(self._jsparkSession.sql(sqlQuery), self)
+        finally:
+            if len(kwargs) > 0:
+                formatter.clear()
+
+    def table(self, tableName: str) -> DataFrame:
         """Returns the specified table as a :class:`DataFrame`.
 
         .. versionadded:: 2.0.0
@@ -738,10 +1052,10 @@ class SparkSession(SparkConversionMixin):
         >>> sorted(df.collect()) == sorted(df2.collect())
         True
         """
-        return DataFrame(self._jsparkSession.table(tableName), self._wrapped)
+        return DataFrame(self._jsparkSession.table(tableName), self)
 
     @property
-    def read(self):
+    def read(self) -> DataFrameReader:
         """
         Returns a :class:`DataFrameReader` that can be used to read data
         in as a :class:`DataFrame`.
@@ -752,10 +1066,10 @@ class SparkSession(SparkConversionMixin):
         -------
         :class:`DataFrameReader`
         """
-        return DataFrameReader(self._wrapped)
+        return DataFrameReader(self)
 
     @property
-    def readStream(self):
+    def readStream(self) -> DataStreamReader:
         """
         Returns a :class:`DataStreamReader` that can be used to read data streams
         as a streaming :class:`DataFrame`.
@@ -770,10 +1084,10 @@ class SparkSession(SparkConversionMixin):
         -------
         :class:`DataStreamReader`
         """
-        return DataStreamReader(self._wrapped)
+        return DataStreamReader(self)
 
     @property
-    def streams(self):
+    def streams(self) -> "StreamingQueryManager":
         """Returns a :class:`StreamingQueryManager` that allows managing all the
         :class:`StreamingQuery` instances active on `this` context.
 
@@ -788,15 +1102,17 @@ class SparkSession(SparkConversionMixin):
         :class:`StreamingQueryManager`
         """
         from pyspark.sql.streaming import StreamingQueryManager
+
         return StreamingQueryManager(self._jsparkSession.streams())
 
     @since(2.0)
-    def stop(self):
-        """Stop the underlying :class:`SparkContext`.
-        """
+    def stop(self) -> None:
+        """Stop the underlying :class:`SparkContext`."""
         from pyspark.sql.context import SQLContext
+
         self._sc.stop()
         # We should clean the default session up. See SPARK-23228.
+        assert self._jvm is not None
         self._jvm.SparkSession.clearDefaultSession()
         self._jvm.SparkSession.clearActiveSession()
         SparkSession._instantiatedSession = None
@@ -804,14 +1120,19 @@ class SparkSession(SparkConversionMixin):
         SQLContext._instantiatedContext = None
 
     @since(2.0)
-    def __enter__(self):
+    def __enter__(self) -> "SparkSession":
         """
         Enable 'with SparkSession.builder.(...).getOrCreate() as session: app' syntax.
         """
         return self
 
     @since(2.0)
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
         """
         Enable 'with SparkSession.builder.(...).getOrCreate() as session: app' syntax.
 
@@ -820,7 +1141,7 @@ class SparkSession(SparkConversionMixin):
         self.stop()
 
 
-def _test():
+def _test() -> None:
     import os
     import doctest
     from pyspark.context import SparkContext
@@ -830,20 +1151,26 @@ def _test():
     os.chdir(os.environ["SPARK_HOME"])
 
     globs = pyspark.sql.session.__dict__.copy()
-    sc = SparkContext('local[4]', 'PythonTest')
-    globs['sc'] = sc
-    globs['spark'] = SparkSession(sc)
-    globs['rdd'] = rdd = sc.parallelize(
-        [Row(field1=1, field2="row1"),
-         Row(field1=2, field2="row2"),
-         Row(field1=3, field2="row3")])
-    globs['df'] = rdd.toDF()
+    sc = SparkContext("local[4]", "PythonTest")
+    globs["sc"] = sc
+    globs["spark"] = SparkSession(sc)
+    globs["rdd"] = rdd = sc.parallelize(
+        [
+            Row(field1=1, field2="row1"),
+            Row(field1=2, field2="row2"),
+            Row(field1=3, field2="row3"),
+        ]
+    )
+    globs["df"] = rdd.toDF()
     (failure_count, test_count) = doctest.testmod(
-        pyspark.sql.session, globs=globs,
-        optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
-    globs['sc'].stop()
+        pyspark.sql.session,
+        globs=globs,
+        optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE,
+    )
+    globs["sc"].stop()
     if failure_count:
         sys.exit(-1)
 
+
 if __name__ == "__main__":
     _test()
