@@ -17,8 +17,6 @@
 
 package org.apache.spark.sql.execution.arrow
 
-import org.apache.arrow.vector.IntervalDayVector
-
 import org.apache.spark.SparkFunSuite
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.util._
@@ -30,12 +28,12 @@ class ArrowWriterSuite extends SparkFunSuite {
 
   test("simple") {
     def check(dt: DataType, data: Seq[Any], timeZoneId: String = null): Unit = {
-      val avroDatatype = dt match {
+      val datatype = dt match {
         case _: DayTimeIntervalType => DayTimeIntervalType()
         case _: YearMonthIntervalType => YearMonthIntervalType()
         case tpe => tpe
       }
-      val schema = new StructType().add("value", avroDatatype, nullable = true)
+      val schema = new StructType().add("value", datatype, nullable = true)
       val writer = ArrowWriter.create(schema, timeZoneId)
       assert(writer.schema === schema)
 
@@ -61,6 +59,7 @@ class ArrowWriterSuite extends SparkFunSuite {
             case BinaryType => reader.getBinary(rowId)
             case DateType => reader.getInt(rowId)
             case TimestampType => reader.getLong(rowId)
+            case TimestampNTZType => reader.getLong(rowId)
             case _: YearMonthIntervalType => reader.getInt(rowId)
             case _: DayTimeIntervalType => reader.getLong(rowId)
           }
@@ -81,6 +80,7 @@ class ArrowWriterSuite extends SparkFunSuite {
     check(BinaryType, Seq("a".getBytes(), "b".getBytes(), null, "d".getBytes()))
     check(DateType, Seq(0, 1, 2, null, 4))
     check(TimestampType, Seq(0L, 3.6e9.toLong, null, 8.64e10.toLong), "America/Los_Angeles")
+    check(TimestampNTZType, Seq(0L, 3.6e9.toLong, null, 8.64e10.toLong))
     check(NullType, Seq(null, null, null))
     DataTypeTestUtils.yearMonthIntervalTypes
       .foreach(check(_, Seq(null, 0, 1, -1, Int.MaxValue, Int.MinValue)))
@@ -88,30 +88,6 @@ class ArrowWriterSuite extends SparkFunSuite {
       Seq(null, 0L, 1000L, -1000L, (Long.MaxValue - 807L), (Long.MinValue + 808L))))
   }
 
-  test("long overflow for DayTimeIntervalType")
-  {
-    val schema = new StructType().add("value", DayTimeIntervalType(), nullable = true)
-    val writer = ArrowWriter.create(schema, null)
-    val reader = new ArrowColumnVector(writer.root.getFieldVectors().get(0))
-    val valueVector = writer.root.getFieldVectors().get(0).asInstanceOf[IntervalDayVector]
-
-    valueVector.set(0, 106751992, 0)
-    valueVector.set(1, 106751991, Int.MaxValue)
-
-    // first long overflow for test Math.multiplyExact()
-    val msg = intercept[java.lang.ArithmeticException] {
-      reader.getLong(0)
-    }.getMessage
-    assert(msg.equals("long overflow"))
-
-    // second long overflow for test Math.addExact()
-    val msg1 = intercept[java.lang.ArithmeticException] {
-      reader.getLong(1)
-    }.getMessage
-    assert(msg1.equals("long overflow"))
-    writer.root.close()
-  }
-
   test("get multiple") {
     def check(dt: DataType, data: Seq[Any], timeZoneId: String = null): Unit = {
       val avroDatatype = dt match {
@@ -139,6 +115,7 @@ class ArrowWriterSuite extends SparkFunSuite {
         case DoubleType => reader.getDoubles(0, data.size)
         case DateType => reader.getInts(0, data.size)
         case TimestampType => reader.getLongs(0, data.size)
+        case TimestampNTZType => reader.getLongs(0, data.size)
         case _: YearMonthIntervalType => reader.getInts(0, data.size)
         case _: DayTimeIntervalType => reader.getLongs(0, data.size)
       }
@@ -155,6 +132,7 @@ class ArrowWriterSuite extends SparkFunSuite {
     check(DoubleType, (0 until 10).map(_.toDouble))
     check(DateType, (0 until 10))
     check(TimestampType, (0 until 10).map(_ * 4.32e10.toLong), "America/Los_Angeles")
+    check(TimestampNTZType, (0 until 10).map(_ * 4.32e10.toLong))
     DataTypeTestUtils.yearMonthIntervalTypes.foreach(check(_, (0 until 14)))
     DataTypeTestUtils.dayTimeIntervalTypes.foreach(check(_, (-10 until 10).map(_ * 1000.toLong)))
   }
