@@ -173,7 +173,7 @@ class HiveThriftServer2ListenerSuite extends SparkFunSuite with BeforeAndAfter {
     if (live) {
       val server = mock(classOf[HiveThriftServer2], RETURNS_SMART_NULLS)
       val listener = new HiveThriftServer2Listener(kvstore, sparkConf, Some(server))
-      (new HiveThriftServer2AppStatusStore(kvstore, Some(listener)), listener)
+      (new HiveThriftServer2AppStatusStore(kvstore), listener)
     } else {
       (new HiveThriftServer2AppStatusStore(kvstore),
         new HiveThriftServer2Listener(kvstore, sparkConf, None, false))
