@@ -20,6 +20,8 @@ package org.apache.spark.sql.hive.orc
 import java.io.File
 
 import com.google.common.io.Files
+import org.apache.hadoop.fs.Path
+import org.apache.orc.OrcConf
 
 import org.apache.spark.sql.{AnalysisException, Row}
 import org.apache.spark.sql.catalyst.TableIdentifier
@@ -50,13 +52,8 @@ class HiveOrcQuerySuite extends OrcQueryTest with TestHiveSingleton {
           val emptyDF = Seq.empty[(Int, String)].toDF("key", "value").coalesce(1)
           emptyDF.createOrReplaceTempView("empty")
 
-          // This creates 1 empty ORC file with Hive ORC SerDe.  We are using this trick because
-          // Spark SQL ORC data source always avoids write empty ORC files.
-          spark.sql(
-            s"""INSERT INTO TABLE empty_orc
-               |SELECT key, value FROM empty
-             """.stripMargin)
-
+          val zeroPath = new Path(path, "zero.orc")
+          zeroPath.getFileSystem(spark.sessionState.newHadoopConf()).create(zeroPath)
           val errorMessage = intercept[AnalysisException] {
             spark.read.orc(path)
           }.getMessage
@@ -254,7 +251,6 @@ class HiveOrcQuerySuite extends OrcQueryTest with TestHiveSingleton {
         withSQLConf(HiveUtils.CONVERT_METASTORE_ORC.key -> "true",
           HiveUtils.CONVERT_INSERTING_PARTITIONED_TABLE.key -> conversion) {
           withTable("dummy_orc_partitioned") {
-            spark.sessionState.refreshTable("dummy_orc_partitioned")
             spark.sql(
               s"""
                  |CREATE TABLE dummy_orc_partitioned(key INT, value STRING)
@@ -313,4 +309,74 @@ class HiveOrcQuerySuite extends OrcQueryTest with TestHiveSingleton {
       }
     }
   }
+
+  test("SPARK-32864: Support ORC forced positional evolution") {
+    Seq("native", "hive").foreach { orcImpl =>
+      Seq(true, false).foreach { forcePositionalEvolution =>
+        Seq(true, false).foreach { convertMetastore =>
+          withSQLConf(SQLConf.ORC_IMPLEMENTATION.key -> orcImpl,
+            OrcConf.FORCE_POSITIONAL_EVOLUTION.getAttribute -> forcePositionalEvolution.toString,
+            HiveUtils.CONVERT_METASTORE_ORC.key -> convertMetastore.toString) {
+            withTempPath { f =>
+              val path = f.getCanonicalPath
+              Seq[(Integer, Integer)]((1, 2), (3, 4), (5, 6), (null, null))
+                .toDF("c1", "c2").write.orc(path)
+              val correctAnswer = Seq(Row(1, 2), Row(3, 4), Row(5, 6), Row(null, null))
+              checkAnswer(spark.read.orc(path), correctAnswer)
+
+              withTable("t") {
+                sql(s"CREATE EXTERNAL TABLE t(c3 INT, c2 INT) STORED AS ORC LOCATION '$path'")
+
+                val expected = if (forcePositionalEvolution) {
+                  correctAnswer
+                } else {
+                  Seq(Row(null, 2), Row(null, 4), Row(null, 6), Row(null, null))
+                }
+
+                checkAnswer(spark.table("t"), expected)
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+  test("SPARK-32864: Support ORC forced positional evolution with partitioned table") {
+    Seq("native", "hive").foreach { orcImpl =>
+      Seq(true, false).foreach { forcePositionalEvolution =>
+        Seq(true, false).foreach { convertMetastore =>
+          withSQLConf(SQLConf.ORC_IMPLEMENTATION.key -> orcImpl,
+            OrcConf.FORCE_POSITIONAL_EVOLUTION.getAttribute -> forcePositionalEvolution.toString,
+            HiveUtils.CONVERT_METASTORE_ORC.key -> convertMetastore.toString) {
+            withTempPath { f =>
+              val path = f.getCanonicalPath
+              Seq[(Integer, Integer, Integer)]((1, 2, 1), (3, 4, 2), (5, 6, 3), (null, null, 4))
+                .toDF("c1", "c2", "p").write.partitionBy("p").orc(path)
+              val correctAnswer = Seq(Row(1, 2, 1), Row(3, 4, 2), Row(5, 6, 3), Row(null, null, 4))
+              checkAnswer(spark.read.orc(path), correctAnswer)
+
+              withTable("t") {
+                sql(
+                  s"""
+                     |CREATE EXTERNAL TABLE t(c3 INT, c2 INT)
+                     |PARTITIONED BY (p int)
+                     |STORED AS ORC
+                     |LOCATION '$path'
+                     |""".stripMargin)
+                sql("MSCK REPAIR TABLE t")
+                val expected = if (forcePositionalEvolution) {
+                  correctAnswer
+                } else {
+                  Seq(Row(null, 2, 1), Row(null, 4, 2), Row(null, 6, 3), Row(null, null, 4))
+                }
+
+                checkAnswer(spark.table("t"), expected)
+              }
+            }
+          }
+        }
+      }
+    }
+  }
 }
